@@ -19,34 +19,73 @@
 from dataclasses import dataclass
 from enum import Enum
 from io import IOBase
-from typing import Any, Dict, List, Optional, Type, Union
+from typing import (
+    Any,
+    Callable,
+    ClassVar,
+    Dict,
+    List,
+    NoReturn,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    get_type_hints,
+)
 import base64
 import datetime
 import json
 
 # Third Party
 from google.protobuf import json_format
-from google.protobuf.descriptor import Descriptor, FieldDescriptor, OneofDescriptor
+from google.protobuf.descriptor import (
+    Descriptor,
+    EnumDescriptor,
+    FieldDescriptor,
+    OneofDescriptor,
+)
 from google.protobuf.internal import type_checkers as proto_type_checkers
 from google.protobuf.message import Message as ProtoMessageType
 
 # First Party
+from py_to_proto.compat_annotated import Annotated, get_args, get_origin
 import alog
 
 # Local
 from ..exceptions import error_handler
 from . import enums, json_dict, timestamp
 
+# if TYPE_CHECKING: # TODO: uncommenting this breaks `tox -e imports` because of a circular import
+#     # Local
+#     from caikit.core.data_model.data_backends import DataModelBackendBase
+#     from caikit.interfaces.common.data_model.file import File
+
 # metaclass-generated field members cannot be detected by pylint
 # pylint: disable=no-member
 # pylint: disable=too-many-lines
 
 
 log = alog.use_channel("DATAM")
-error = error_handler.get(log)
+error: Callable[..., NoReturn] = error_handler.get(log)
 
 
 class _DataBaseMetaClass(type):
+    fields: Tuple
+    full_name: str
+    fields_enum_map: Dict  # {}
+    fields_enum_rev: Dict  # {}
+    _fields_oneofs_map: Dict  # {}
+    _fields_to_oneof: Dict  # {}
+    _fields_to_type: Dict  # {}
+    _fields_map: Tuple  # ()
+    _fields_message: Tuple  # ()
+    _fields_message_repeated: Tuple  # ()
+    _fields_enum: Tuple  # ()
+    _fields_enum_repeated: Tuple  # ()
+    _fields_primitive: Tuple  # ()
+    _fields_primitive_repeated: Tuple  # ()
+    _proto_class: ClassVar[Type[ProtoMessageType]]
+
     """Meta class for all structures in the data model."""
 
     # store a registry of all classes that use this metaclass, i.e.,
@@ -103,6 +142,7 @@ class _DataBaseMetaClass(type):
         attrs["fields_enum_rev"] = {}
         attrs["_fields_oneofs_map"] = {}
         attrs["_fields_to_oneof"] = {}
+        attrs["_fields_to_type"] = {}
         attrs["_fields_map"] = ()
         attrs["_fields_message"] = ()
         attrs["_fields_message_repeated"] = ()
@@ -136,7 +176,7 @@ class _DataBaseMetaClass(type):
             # Otherwise, we need to get the fields from a "special" attribute
             else:
                 fields = attrs.pop(mcs._FWD_DECL_FIELDS, None)
-                log.debug4(
+                log.debug4(  # type: ignore
                     "Using dataclass forward declaration fields %s for %s", fields, name
                 )
                 error.value_check(
@@ -170,7 +210,7 @@ class _DataBaseMetaClass(type):
         return instance
 
     @classmethod
-    def parse_proto_descriptor(mcs, cls):
+    def parse_proto_descriptor(mcs, cls):  # pyright: ignore[reportSelfClsParameterName]
         """Encapsulate the logic for parsing the protobuf descriptor here. This
         allows the parsing to be done as a post-process after metaclass
         initialization
@@ -304,17 +344,17 @@ class _DataBaseMetaClass(type):
         # If there is not already an __init__ function defined, make one
         current_init = cls.__init__
         if current_init is None or current_init is DataBase.__init__:
-            setattr(cls, "__init__", mcs._make_init(cls.fields))
+            cls.__init__ = mcs._make_init(cls.fields)
 
         # Check DataBase for file handlers
-        setattr(
-            cls,
-            "supports_file_operations",
-            cls.to_file != DataBase.to_file and cls.from_file != DataBase.from_file,
+        cls.supports_file_operations = (
+            cls.to_file != DataBase.to_file and cls.from_file != DataBase.from_file
         )
 
     @classmethod
-    def _make_property_getter(mcs, field, oneof_name=None):
+    def _make_property_getter(
+        mcs, field, oneof_name=None  # pyright: ignore[reportSelfClsParameterName]
+    ):
         """This helper creates an @property attribute getter for the given field
 
         NOTE: This needs to live as a standalone function in order for the given
@@ -340,7 +380,7 @@ class _DataBaseMetaClass(type):
                 )
             attr_val = backend.get_attribute(self.__class__, field)
             if isinstance(attr_val, self.__class__.OneofFieldVal):
-                log.debug2("Got a OneofFieldVal from the backend")
+                log.debug2("Got a OneofFieldVal from the backend")  # type: ignore
                 assert field in self.__class__._fields_oneofs_map
                 self._get_which_oneof_dict()[field] = attr_val.which_oneof
                 attr_val = attr_val.val
@@ -434,12 +474,19 @@ class _DataBaseMetaClass(type):
                     ):
                         setattr(self, field_name, None)
 
+            # Add type information for all fields. Do this during init to
+            # allow for forward refs to be imported
+            for field in cls.fields:
+                cls._fields_to_type[field] = cls._get_type_for_field(field)
+
         # Set docstring to the method explicitly
-        setattr(__init__, "__doc__", docstring)
+        __init__.___doc__ = docstring
         return __init__
 
     @classmethod
-    def _sorted_oneof_field_names(mcs, oneof: OneofDescriptor) -> List[str]:
+    def _sorted_oneof_field_names(
+        mcs, oneof: OneofDescriptor  # pyright: ignore[reportSelfClsParameterName]
+    ) -> List[str]:
         """Helper to get the list of oneof fields while ensuring field names are
         sorted such that bool < int < float. This ensures that when iterating
         fields for which_oneof inference, lower-precedence types take
@@ -464,6 +511,13 @@ class DataBase(metaclass=_DataBaseMetaClass):
         defined in the interface definitions.  If not, an exception will be thrown at runtime.
     """
 
+    # Class constant used to identify protobuf types that are handled with
+    # special logic in the to/from proto conversions
+    PROTO_CONVERSION_SPECIAL_TYPES = [
+        timestamp.TIMESTAMP_PROTOBUF_NAME,
+        json_dict.STRUCT_PROTOBUF_NAME,
+    ]
+
     @dataclass
     class OneofFieldVal:
         """Helper struct that backends can use to return information about
@@ -499,28 +553,30 @@ class DataBase(metaclass=_DataBaseMetaClass):
         return cls._proto_class
 
     @classmethod
-    def get_field_message_type(cls, field_name: str) -> Optional[Type["DataBase"]]:
-        """Get the data model class for the given field if the field is a
-        message or a repeated message
+    def get_field_message_type(cls, field_name: str) -> Optional[type]:
+        """Get the python type for the given field. This function relies on the
+        metaclass to fill cls._fields_to_type. This is to avoid costly
+        computation during runtime
 
         Args:
             field_name (str): Field name to check (AttributeError raised if name
                 is invalid)
 
         Returns:
-            data_model_type:  Type[DataBase]
+            field_type:  type
                 The data model class type for the given field
         """
+
+        # Dataclass look ups are fast so keep them in to retain interface compatibility
         if field_name not in cls.fields:
             raise AttributeError(f"Invalid field {field_name}")
-        if (
-            field_name in cls._fields_message
-            or field_name in cls._fields_message_repeated
-        ):
-            return cls.get_class_for_proto(
-                cls.get_proto_class().DESCRIPTOR.fields_by_name[field_name].message_type
-            )
-        return None
+
+        # If field_name has not been cached then perform lookup and
+        # save result
+        if field_name not in cls._fields_to_type:
+            cls._fields_to_type[field_name] = cls._get_type_for_field(field_name)
+
+        return cls._fields_to_type.get(field_name)
 
     @classmethod
     def from_backend(cls, backend):
@@ -529,7 +585,7 @@ class DataBase(metaclass=_DataBaseMetaClass):
         return instance
 
     @property
-    def backend(self) -> Optional["DataModelBackendBase"]:
+    def backend(self) -> Optional["DataModelBackendBase"]:  # type: ignore # noqa: F821 # see TYPE_CHECKING note at the top
         return getattr(self, _DataBaseMetaClass._BACKEND_ATTR, None)
 
     def which_oneof(self, oneof_name: str) -> Optional[str]:
@@ -577,6 +633,64 @@ class DataBase(metaclass=_DataBaseMetaClass):
             which_oneof = getattr(self, _DataBaseMetaClass._WHICH_ONEOF_ATTR)
         return which_oneof
 
+    @classmethod
+    def _get_type_for_field(cls, field_name: str) -> type:
+        """Helper class method to return the type hint for a particular field"""
+        cls_type_hints = get_type_hints(cls)
+        if type_hint := cls_type_hints.get(field_name):
+
+            # If type is optional or a list then return internal type
+            type_args = get_args(type_hint)
+            if (
+                get_origin(type_hint) == Union
+                and type_args
+                == (
+                    type_args[0],
+                    type(None),
+                )
+                or get_origin(type_hint) in [list, List]
+            ):
+                type_hint = type_args[0]
+
+            # If type is Annotated then get the actual type
+            if get_origin(type_hint) == Annotated:
+                type_hint = get_args(type_hint)[0]
+
+            return type_hint
+
+        fd = cls._proto_class.DESCRIPTOR.fields_by_name.get(field_name)
+        if not fd:
+            raise ValueError(f"Unknown field: {field_name}")
+
+        # Convert the fd type into python
+        if fd.type == fd.TYPE_MESSAGE:
+            return cls.get_class_for_proto(fd.message_type)
+        elif fd.type == fd.TYPE_ENUM:
+            return cls.get_class_for_proto(fd.enum_type)
+        elif fd.type == fd.TYPE_BOOL:
+            return bool
+        elif fd.type == fd.TYPE_BYTES:
+            return bytes
+        elif fd.type == fd.TYPE_STRING:
+            return str
+        elif fd.type in [
+            fd.TYPE_FIXED32,
+            fd.TYPE_FIXED64,
+            fd.TYPE_INT32,
+            fd.TYPE_INT64,
+            fd.TYPE_SFIXED32,
+            fd.TYPE_SFIXED64,
+            fd.TYPE_SINT32,
+            fd.TYPE_SINT64,
+            fd.TYPE_UINT32,
+            fd.TYPE_UINT64,
+        ]:
+            return int
+        elif fd.type in [fd.TYPE_FLOAT, fd.TYPE_DOUBLE]:
+            return float
+
+        raise ValueError(f"Unknown proto type: {fd.type}")
+
     @classmethod
     def _is_valid_type_for_field(cls, field_name: str, val: Any) -> bool:
         """Check whether the given value is valid for the given field"""
@@ -606,7 +720,7 @@ class DataBase(metaclass=_DataBaseMetaClass):
             and field_descriptor.message_type == val.get_proto_class().DESCRIPTOR
         ) or (
             isinstance(val, Enum)
-            and field_descriptor.enum_type == val.get_proto_class().DESCRIPTOR
+            and field_descriptor.enum_type == val.get_proto_class().DESCRIPTOR  # type: ignore
         ):
             return True
 
@@ -628,7 +742,7 @@ class DataBase(metaclass=_DataBaseMetaClass):
             return isinstance(val, bytes)
 
         # If it's a primitive, use protobuf type checkers
-        checker = proto_type_checkers.GetTypeChecker(field_descriptor)
+        checker = proto_type_checkers.GetTypeChecker(field_descriptor)  # type: ignore
         try:
             checker.CheckValue(val)
             return True
@@ -730,7 +844,7 @@ class DataBase(metaclass=_DataBaseMetaClass):
                                 "caikit.interfaces.common.data_model"
                             )
                         ):
-                            kwargs[oneof] = getattr(contained_obj, "values")
+                            kwargs[oneof] = contained_obj.values  # type: ignore
                         else:
                             kwargs[oneof] = contained_obj
                     else:
@@ -900,9 +1014,9 @@ class DataBase(metaclass=_DataBaseMetaClass):
                     seq_dm = subproto.__class__
                     try:
                         subproto.CopyFrom(seq_dm(values=attr))
-                        log.debug4("Successfully fill proto for %s", field)
+                        log.debug4("Successfully fill proto for %s", field)  # type: ignore
                     except TypeError:
-                        log.debug4("not the correct union list type")
+                        log.debug4("not the correct union list type")  # type: ignore
                 else:
                     attr.fill_proto(subproto)
 
@@ -936,7 +1050,7 @@ class DataBase(metaclass=_DataBaseMetaClass):
         fields_to_dict = []
         for field in self.fields:
             if (
-                not field in self._fields_to_oneof
+                field not in self._fields_to_oneof
                 or self.which_oneof(self._fields_to_oneof[field]) == field
             ):
                 fields_to_dict.append(field)
@@ -970,7 +1084,7 @@ class DataBase(metaclass=_DataBaseMetaClass):
         """Convert to a json representation."""
 
         def _default_serialization_overrides(obj):
-            """Default handler for nonserializable objects; currently this only handles
+            """Default handler for non-serializable objects; currently this only handles
             - bytes
             - datetime.datetime
             """
@@ -986,7 +1100,9 @@ class DataBase(metaclass=_DataBaseMetaClass):
 
         return json.dumps(self.to_dict(), **kwargs)
 
-    def to_file(self, file_obj: IOBase) -> Optional["File"]:
+    def to_file(
+        self, file_obj: IOBase
+    ) -> Optional["File"]:  # type: ignore # noqa: F821 # see TYPE_CHECKING note at the top
         """Export a DataBaseObject into a file-like object `file_obj`. If the DataBase object
         has requirements around file name or file type it can return them via
         the optional "File" return object
@@ -1065,7 +1181,7 @@ class DataBase(metaclass=_DataBaseMetaClass):
 
     @staticmethod
     def get_class_for_proto(
-        proto: Union[Descriptor, ProtoMessageType]
+        proto: Union[Descriptor, FieldDescriptor, EnumDescriptor, ProtoMessageType]
     ) -> Type["DataBase"]:
         """Look up the data model class corresponding to the given protobuf
 
@@ -1082,12 +1198,14 @@ class DataBase(metaclass=_DataBaseMetaClass):
         error.type_check(
             "<COR46446770E>",
             Descriptor,
+            FieldDescriptor,
+            EnumDescriptor,
             ProtoMessageType,
             proto=proto,
         )
         proto_full_name = (
             proto.full_name
-            if isinstance(proto, Descriptor)
+            if isinstance(proto, (Descriptor, FieldDescriptor, EnumDescriptor))
             else proto.DESCRIPTOR.full_name
         )
         cls = _DataBaseMetaClass.class_registry.get(proto_full_name)
