@@ -17,6 +17,7 @@
 """
 # Standard
 from collections.abc import Iterable
+from functools import cached_property
 from glob import glob
 from io import UnsupportedOperation
 from typing import Dict, Generic, List, Tuple, TypeVar, Union
@@ -47,6 +48,7 @@ error = error_handler.get(log)
 
 T = TypeVar("T")
 
+
 # ghart: These public methods are all needed. This class is essentially its own factory, so these
 # are all the different ways of coercing different data sources into a common stream class
 # pylint: disable=too-many-public-methods
@@ -103,7 +105,6 @@ class DataStream(Generic[T]):
 
         self.generator_func = generator_func
         self.generator_args, self.generator_kwargs = args, kwargs
-        self._length = None
 
     @classmethod
     def from_iterable(cls, data: typing.Iterable[T]) -> "DataStream[T]":
@@ -314,15 +315,14 @@ class DataStream(Generic[T]):
     @classmethod
     def _from_csv_generator(cls, filename, skip, *csv_args, **csv_kwargs):
         # open the csv file (closure around `filename`)
-        with open(filename, mode="r", encoding="utf8") as fh:
+        with open(filename, encoding="utf8") as fh:
             # skip lines if requested
             for _ in range(skip):
                 # pylint: disable=stop-iteration-return
                 next(fh)
 
             # for each line of the csv file, yield a list
-            for line in csv.reader(fh, *csv_args, **csv_kwargs):
-                yield line
+            yield from csv.reader(fh, *csv_args, **csv_kwargs)
 
     @classmethod
     def from_header_csv(cls, filename: str, *args, **kwargs) -> "DataStream[Dict]":
@@ -368,7 +368,7 @@ class DataStream(Generic[T]):
     @classmethod
     def _from_header_csv_generator(cls, filename, *csv_args, **csv_kwargs):
         # open the csv file (closure around `filename`)
-        with open(filename, mode="r", encoding="utf8") as fh:
+        with open(filename, encoding="utf8") as fh:
             yield from cls._from_header_csv_buffer_generator(
                 fh, *csv_args, **csv_kwargs
             )
@@ -384,8 +384,7 @@ class DataStream(Generic[T]):
                 RuntimeError("File handler for csv with header not seekable"),
             )
         # for each line of the csv file, yield a dict
-        for line in csv.DictReader(fh, *csv_args, **csv_kwargs):
-            yield line
+        yield from csv.DictReader(fh, *csv_args, **csv_kwargs)
 
     @classmethod
     def from_txt(cls, filename: str) -> "DataStream[str]":
@@ -425,7 +424,7 @@ class DataStream(Generic[T]):
     @classmethod
     def _from_txt_generator(cls, filename):
         # open the file (closure around `filename`)
-        with open(filename, mode="r", encoding="utf8") as fh:
+        with open(filename, encoding="utf8") as fh:
             # for each line of the file
             for line in fh:
                 # strip new lines and carriage returns and yield the line
@@ -573,8 +572,7 @@ class DataStream(Generic[T]):
         for filename in glob(os.path.join(dirname, "*.csv")):
             data_stream_list.append(cls.from_header_csv(filename=filename))
         # yield the combined data item once flattened
-        for data_item in DataStream.chain(data_stream_list).flatten():
-            yield data_item
+        yield from DataStream.chain(data_stream_list).flatten()
 
     @classmethod
     def from_jsonl_collection(cls, dirname: str) -> "DataStream[Dict]":
@@ -602,8 +600,7 @@ class DataStream(Generic[T]):
         for filename in glob(os.path.join(dirname, "*.jsonl")):
             data_stream_list.append(cls.from_jsonl(filename=filename))
         # yield the combined data item once flattened
-        for data_item in DataStream.chain(data_stream_list).flatten():
-            yield data_item
+        yield from DataStream.chain(data_stream_list).flatten()
 
     @classmethod
     def from_multipart_file(cls, filename: str) -> "DataStream[JsonDictValue]":
@@ -806,8 +803,7 @@ class DataStream(Generic[T]):
 
         def generator_func():
             for inner_stream in self:
-                for data_item in inner_stream:
-                    yield data_item
+                yield from inner_stream
 
         return DataStream(generator_func)
 
@@ -995,15 +991,21 @@ class DataStream(Generic[T]):
         return generator
 
     def __len__(self):
-        """Return the number of data items contained in this data stream.  This requires that the
-        data stream be iterated over, which may be time consuming.  This value is then stored
-        internally so that subsequent calls do not iterate over the data stream again.
-        """
-        if self._length is None:
-            self._length = sum(1 for data_item in self)
-
+        """See property method self._length"""
         return self._length
 
+    @cached_property
+    def _length(self):
+        """Return the number of data items contained in this data stream.  This requires that the
+        data stream be iterated over, which may be time-consuming.  This value is then stored
+        internally so that subsequent calls do not iterate over the data stream again.
+
+        This is implemented as a cached_property so that subclasses of DataStream which implement
+        their own __getstate__ and __setstate__ do not have to account for the existence of
+        self._length
+        """
+        return sum(1 for _ in self)
+
     def __or__(self, module):
         """Feed this data stream into the `.stream` method of a module.  This is syntactic sugar
         that allows modules to be chained like `data_stream | module1 | module2` rather than the
