@@ -19,18 +19,29 @@
 # Standard
 from collections.abc import Iterable
 from types import GeneratorType
-from typing import Any
+from typing import TYPE_CHECKING, Dict, NoReturn, Optional, Type, Union
 import os
+import typing
+
+# First Party
+from alog.protocols import LoggerProtocol
 
 # Local
 from caikit.config import get_config
 
+if TYPE_CHECKING:
+    # Standard
+    from logging import Logger
+
+    # Third Party
+    from _typeshed import FileDescriptorOrPath
+
 # dictionary mapping string log channel name to error handler instances
 # there is only one error handler instance for each log channel name
-_error_handlers = {}
+_error_handlers: Dict[str, "ErrorHandler"] = {}
 
 
-def get(log_chan):
+def get(log_chan: Union["Logger", LoggerProtocol]):
     """Get an error handler associated with a given alog log channel.  The same error handler will
     be returned if this function is called repeatedly with the same log channel.
 
@@ -53,7 +64,7 @@ class ErrorHandler:
     the `.log_raise` method.
     """
 
-    def __init__(self, log_chan):
+    def __init__(self, log_chan: Union["Logger", LoggerProtocol]):
         """Create a new error handler that provides reusable error checking and automatic logging.
 
         Args:
@@ -62,7 +73,7 @@ class ErrorHandler:
         """
         self.log_chan = log_chan
 
-    def _handle_exception_messages(self, log_code, exception):
+    def _handle_exception_messages(self, log_code: str, exception: Exception):
         """Handle number of exception log messages to avoid overflows"""
         # increment the log message counter attribute or add it if not present
         if hasattr(exception, "_caikit_core_nexception_log_messages"):
@@ -93,7 +104,12 @@ class ErrorHandler:
                 ),
             )
 
-    def log_raise(self, log_code, exception, root_exception=None):
+    def log_raise(
+        self,
+        log_code: str,
+        exception: Exception,
+        root_exception: Optional[Exception] = None,
+    ) -> NoReturn:
         """Log an exception with a log code and then re-raise it.  Using this instead of simply
         using the `raise` keyword with your exceptions will ensure that log message is emitted on
         the `error` level for the log channel associated with this handler.  This is invaluable for
@@ -132,7 +148,9 @@ class ErrorHandler:
     # calling an error handler is equivalent to calling the `.log_raise` method
     __call__ = log_raise
 
-    def type_check(self, log_code, *types, allow_none=False, **variables):
+    def type_check(
+        self, log_code: str, *types: Type, allow_none: bool = False, **variables: object
+    ) -> None:
         """Check for acceptable types for a given object.  If the type check fails, a log message
         will be emitted at the error level on the log channel associated with this handler and a
         `TypeError` exception will be raised with an appropriate message.  This check should be used
@@ -200,7 +218,13 @@ class ErrorHandler:
                     ),
                 )
 
-    def type_check_all(self, log_code, *types, allow_none=False, **variables):
+    def type_check_all(
+        self,
+        log_code: str,
+        *types: Type,
+        allow_none: bool = False,
+        **variables: typing.Iterable[object]
+    ) -> None:
         """This type check is similar to `.type_check` except that it verifies that each variable
         in `**variables` is either a `list` or a `tuple` and then checks that *all* of the items
         they contain are instances of a type in `*types`.  If `allow_none` is set to `True`, then
@@ -277,8 +301,12 @@ class ErrorHandler:
                     )
 
     def subclass_check(
-        self, log_code: str, child_class: Any, *parent_classes, allow_none: bool = False
-    ):
+        self,
+        log_code: str,
+        child_class: Type,
+        *parent_classes: Type,
+        allow_none: bool = False
+    ) -> None:
         """Check that the given child classes are valid types and that they
         derive from the given set of parent classes [issubclass(x, (y, z))]. If
         the subclass check fails, a log message will be emitted at the error
@@ -344,7 +372,7 @@ class ErrorHandler:
                 ),
             )
 
-    def value_check(self, log_code, condition, *args):
+    def value_check(self, log_code: str, condition: bool, *args: object) -> None:
         """Check for acceptable values for a given object.  If this check fails, a log message will
         be emitted at the error level on the log channel associated with this handler and a
         `ValueError` exception will be raised with an appropriate message.  This check should be
@@ -385,7 +413,7 @@ class ErrorHandler:
                 log_code, ValueError("value check failed: {}".format(interpolated_msg))
             )
 
-    def file_check(self, log_code, *file_paths):
+    def file_check(self, log_code: str, *file_paths: "FileDescriptorOrPath") -> None:
         """Check to see if one or more file paths exist and are regular files.  If any do not exist
         or are not files, then a log message will be emitted on the log channel associated with this
         error handler and a `FileNotFoundError` will be raised with an appropriate error message.
@@ -397,8 +425,8 @@ class ErrorHandler:
                 (example generation in `scripts/cor_log_code`) and where `E` is
                 an error level short-code, one of `{'fatal': 'F', 'error': 'E',
                 'warning': 'W', 'info': 'I', 'trace': 'T', 'debug': 'D'}`.
-            *file_paths (str): Variadic argument containing strings specifying
-                the file paths to check.  If any of these file paths does not
+            *file_paths (FileDescriptorOrPath): Variadic argument containing strings
+                specifying the file paths to check.  If any of these file paths does not
                 exist or is not a regular file, then a log message will be
                 emitted and a `FileNotFoundError` will be raised.
         """
@@ -420,7 +448,7 @@ class ErrorHandler:
                     FileNotFoundError("Path `{}` is not a file".format(file_path)),
                 )
 
-    def dir_check(self, log_code, *dir_paths):
+    def dir_check(self, log_code: str, *dir_paths: "FileDescriptorOrPath") -> None:
         """Check to see if one or more directory paths exist and are, in fact, directories.  If any
         do not exist then a `FileNotFoundError` will be raised and if they are not directories then
         a `NotADirectoryError` will be raised.  In either case, a log message will be emitted on the
@@ -433,8 +461,8 @@ class ErrorHandler:
                 (example generation in `scripts/cor_log_code`) and where `E` is
                 an error level short-code, one of `{'fatal': 'F', 'error': 'E',
                 'warning': 'W', 'info': 'I', 'trace': 'T', 'debug': 'D'}`.
-            *dir_paths (str): Variadic argument containing strings specifying
-                the directory paths to check.  If any of these file paths does
+            *dir_paths (FileDescriptorOrPath): Variadic argument containing strings
+                specifying the directory paths to check.  If any of these file paths does
                 not exist or is not a regular file, then a log message will be
                 emitted and a `FileNotFoundError` or `NotADirectoryError` will
                 raised.
@@ -457,7 +485,7 @@ class ErrorHandler:
                     NotADirectoryError("Path `{}` is not a directory".format(dir_path)),
                 )
 
-    def _fqname(self, o) -> str:
+    def _fqname(self, o: object) -> str:
         try:
             class_ = o.__class__
             return ".".join([class_.__module__, class_.__qualname__])
