@@ -13,10 +13,14 @@
 # limitations under the License.
 
 
+# Standard
+from typing import Optional
+
+
 class DataValidationError(Exception):
     """This error is used for data validation problems during training"""
 
-    def __init__(self, reason, item_number=None):
+    def __init__(self, reason: str, item_number: Optional[int] = None):
         if item_number:
             message = "Training data validation failed on item {}. {}".format(
                 item_number, reason
@@ -33,6 +37,6 @@ class DataValidationError(Exception):
         return self._reason
 
     @property
-    def item_number(self) -> int:
+    def item_number(self) -> Optional[int]:
         """The index of the training data item that failed validation. Probably zero indexed"""
         return self._item_number
