@@ -169,9 +169,17 @@ class TaskBase:
         ).items():
             signature_type = signature.parameters[parameter_name]
             if parameter_type != signature_type:
-                if typing.get_origin(
-                    signature_type
-                ) == typing.Union and parameter_type in typing.get_args(signature_type):
+                if typing.get_origin(signature_type) == typing.Union and (
+                    # Either our parameter type is not a union & is part of the union signature
+                    parameter_type in typing.get_args(signature_type)
+                    # Or our parameter type is a union that's a subset of the union signature
+                    or (
+                        typing.get_origin(parameter_type) == typing.Union
+                        and set(typing.get_args(parameter_type)).issubset(
+                            set(typing.get_args(signature_type))
+                        )
+                    )
+                ):
                     continue
                 if input_streaming and cls._is_iterable_type(parameter_type):
                     streaming_type = typing.get_args(parameter_type)[0]
@@ -239,14 +247,13 @@ class TaskBase:
                     return
 
         # Do some streaming checks
-        if output_streaming:
-            if cls._is_iterable_type(output_type):
-                # task_output_type is already guaranteed to be Iterable[T]
-                streaming_type = typing.get_args(task_output_type)[0]
+        if output_streaming and cls._is_iterable_type(output_type):
+            # task_output_type is already guaranteed to be Iterable[T]
+            streaming_type = typing.get_args(task_output_type)[0]
 
-                for iterable_type in typing.get_args(output_type):
-                    if cls._subclass_check(iterable_type, streaming_type):
-                        return
+            for iterable_type in typing.get_args(output_type):
+                if cls._subclass_check(iterable_type, streaming_type):
+                    return
 
         raise TypeError(
             f"Wrong output type for module {module}: "
