@@ -16,7 +16,8 @@ This file contains interfaces required for unions of lists
 """
 
 # Standard
-from typing import List
+from dataclasses import dataclass
+from typing import Any, List
 
 # First Party
 from py_to_proto.dataclass_to_proto import Annotated, FieldNumber
@@ -25,21 +26,32 @@ from py_to_proto.dataclass_to_proto import Annotated, FieldNumber
 from caikit.core.data_model import PACKAGE_COMMON, DataObjectBase, dataobject
 
 
+class Sequence(DataObjectBase):
+    """Base class for all Sequences to enable type checking
+    e.g. isinstance(<>, Sequence)"""
+
+    values: List[Any]
+
+
 @dataobject(PACKAGE_COMMON)
-class IntSequence(DataObjectBase):
+@dataclass
+class IntSequence(Sequence):
     values: Annotated[List[int], FieldNumber(1)]
 
 
 @dataobject(PACKAGE_COMMON)
-class FloatSequence(DataObjectBase):
+@dataclass
+class FloatSequence(Sequence):
     values: Annotated[List[float], FieldNumber(1)]
 
 
 @dataobject(PACKAGE_COMMON)
-class StrSequence(DataObjectBase):
+@dataclass
+class StrSequence(Sequence):
     values: Annotated[List[str], FieldNumber(1)]
 
 
 @dataobject(PACKAGE_COMMON)
-class BoolSequence(DataObjectBase):
+@dataclass
+class BoolSequence(Sequence):
     values: Annotated[List[bool], FieldNumber(1)]
