@@ -25,5 +25,10 @@ class AbortedException(CaikitRuntimeException):
         message: Because Exceptions usually have those
     """
 
-    def __init__(self, message):
+    def __init__(self, message: str = None):
+        if not message:
+            message = (
+                "Work in this thread was aborted by a context manager. "
+                "This is usually due to a client timeout or cancellation."
+            )
         super().__init__(grpc.StatusCode.ABORTED, message)
