@@ -209,6 +209,7 @@ def validate_data_model(
     # this `cdm` was moved here from import-time
     cdm = get_data_model()
     for method in service_descriptor.methods:
+        log.debug("Validating method: %s", method.name)
         # Retrieve the descriptor of the input message for this RPC, and
         # verify that each field of the input message can be translated
         # into a corresponding object of the Caikit Library CDM, and that each
@@ -220,7 +221,6 @@ def validate_data_model(
             # and check to make that it is either a primitive protobufs type or that
             # we have a data model class that we can deserialize the protobufs with
             if not is_protobuf_primitive_field(field):
-
                 if field.message_type and field.message_type.GetOptions().map_entry:
                     log.debug(
                         "<RUN51658878D>",
@@ -231,21 +231,27 @@ def validate_data_model(
                     )
                     continue
 
-                # ... or that we can get the field type name, e.g., RawDocument...
-                field_type = input_proto_msg.fields_by_name[
+                field_message_type = input_proto_msg.fields_by_name[
                     field.name
-                ].message_type.name
+                ].message_type
+                if (
+                    field_message_type.full_name
+                    not in DataBase.PROTO_CONVERSION_SPECIAL_TYPES
+                ):
 
-                # ...and ensuring that we can load a corresponding object from the Caikit* CDM
-                caikit_library_class = validate_caikit_library_class_exists(
-                    cdm, field_type
-                )
+                    # ... or that we can get the field type name, e.g., RawDocument...
+                    field_type = field_message_type.name
 
-                # ...and also ensuring that the Caikit Library CDM class has a `from_proto`
-                # method...
-                validate_caikit_library_class_method_exists(
-                    caikit_library_class, "from_proto"
-                )
+                    # ...and ensuring that we can load a corresponding object from the Caikit* CDM
+                    caikit_library_class = validate_caikit_library_class_exists(
+                        cdm, field_type
+                    )
+
+                    # ...and also ensuring that the Caikit Library CDM class has a `from_proto`
+                    # method...
+                    validate_caikit_library_class_method_exists(
+                        caikit_library_class, "from_proto"
+                    )
             else:
                 log.debug(
                     "<RUN51658879D>",
@@ -261,8 +267,13 @@ def validate_data_model(
         # all Caikit library modules should return well formed "predict" messages
         # from the data model.
         output_class = method.output_type.name
-        caikit_Library_class = validate_caikit_library_class_exists(cdm, output_class)
-        validate_caikit_library_class_method_exists(caikit_Library_class, "to_proto")
+        if method.output_type.full_name not in DataBase.PROTO_CONVERSION_SPECIAL_TYPES:
+            caikit_library_class = validate_caikit_library_class_exists(
+                cdm, output_class
+            )
+            validate_caikit_library_class_method_exists(
+                caikit_library_class, "to_proto"
+            )
 
 
 class ServicePackageStreamWrapper(DataStreamSourceBase):
@@ -400,9 +411,7 @@ def build_caikit_library_request_dict(
 
         # 2. Remove any fields not in the module signature
         absent_field_names = [
-            field
-            for field in kwargs_dict.keys()
-            if field not in module_signature.parameters.keys()
+            field for field in kwargs_dict if field not in module_signature.parameters
         ]
         for absent_field_name in absent_field_names:
             kwargs_dict.pop(absent_field_name)
@@ -410,7 +419,6 @@ def build_caikit_library_request_dict(
         # 3. Handle type conversions
         updated_kwargs = {}
         for field_name, field_value in kwargs_dict.items():
-
             # 3.1 Model Pointers
             if isinstance(field_value, ModelPointer):
                 log.debug2("field %s value is a ModelPointer obj", field_name)
