@@ -3,9 +3,9 @@ This sets up global test configs when pytest starts
 """
 
 # Standard
-from contextlib import contextmanager
+from contextlib import closing, contextmanager
 from functools import partial
-from typing import Dict, Type, Union
+from typing import Dict, List, Optional, Type, Union
 import os
 import shlex
 import socket
@@ -14,19 +14,26 @@ import sys
 import tempfile
 import threading
 import time
+import warnings
 
 # Third Party
 from grpc_health.v1 import health_pb2, health_pb2_grpc
 import grpc
 import pytest
 import requests
+import tls_test_tools
 
 # First Party
+import aconfig
 import alog
 
 # Local
 from caikit.core import MODEL_MANAGER
-from caikit.core.data_model.dataobject import render_dataobject_protos
+from caikit.core.data_model.dataobject import (
+    DataObjectBase,
+    dataobject,
+    render_dataobject_protos,
+)
 from caikit.runtime import http_server
 from caikit.runtime.grpc_server import RuntimeGRPCServer
 from caikit.runtime.model_management.loaded_model import LoadedModel
@@ -35,19 +42,30 @@ from caikit.runtime.service_factory import ServicePackage, ServicePackageFactory
 from caikit.runtime.service_generation.rpcs import TaskPredictRPC
 from caikit.runtime.servicers.global_predict_servicer import GlobalPredictServicer
 from caikit.runtime.servicers.global_train_servicer import GlobalTrainServicer
+from caikit.runtime.servicers.model_runtime_servicer import ModelRuntimeServicerImpl
+from caikit.runtime.work_management.abortable_context import ThreadInterrupter
 from tests.conftest import random_test_id, temp_config
 from tests.fixtures import Fixtures
 
 log = alog.use_channel("TEST-CONFTEST")
 
 
+def get_open_port():
+    """Non-fixture function to get an open port"""
+    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
+        s.bind(("", 0))
+        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+        port = s.getsockname()[1]
+        return port
+
+
 @pytest.fixture
 def open_port():
     """Get an open port on localhost
     Returns:
         int: Available port
     """
-    return _open_port()
+    return get_open_port()
 
 
 @pytest.fixture(scope="session")
@@ -56,7 +74,7 @@ def session_scoped_open_port():
     Returns:
         int: Available port
     """
-    return _open_port()
+    return get_open_port()
 
 
 @pytest.fixture(scope="session")
@@ -65,22 +83,7 @@ def http_session_scoped_open_port():
     Returns:
         int: Available port
     """
-    return _open_port()
-
-
-def _open_port():
-    # TODO: This has obvious problems where the port returned for use by a test is not immediately
-    # put into use, so parallel tests could attempt to use the same port.
-    start = 8888
-    end = start + 1000
-    host = "localhost"
-    for port in range(start, end):
-        with socket.socket() as soc:
-            # soc.connect_ex returns 0 if connection is successful,
-            # indicating the port is in use
-            if soc.connect_ex((host, port)) != 0:
-                # So a non-zero code should mean the port is not currently in use
-                return port
+    return get_open_port()
 
 
 @pytest.fixture(scope="session")
@@ -98,13 +101,18 @@ def sample_inference_service(render_protos) -> ServicePackage:
 
 @pytest.fixture(scope="session")
 def sample_predict_servicer(sample_inference_service) -> GlobalPredictServicer:
-    servicer = GlobalPredictServicer(inference_service=sample_inference_service)
+    interrupter = ThreadInterrupter()
+    interrupter.start()
+    servicer = GlobalPredictServicer(
+        inference_service=sample_inference_service, interrupter=interrupter
+    )
     yield servicer
     # Make sure to not leave the rpc_meter hanging
     # (It does try to clean itself up on destruction, but just to be sure)
     rpc_meter = getattr(servicer, "rpc_meter", None)
     if rpc_meter:
         rpc_meter.end_writer_thread()
+    interrupter.stop()
 
 
 @pytest.fixture(scope="session")
@@ -151,9 +159,7 @@ def runtime_grpc_test_server(open_port, *args, **kwargs):
 
 
 @pytest.fixture(scope="session")
-def runtime_grpc_server(
-    session_scoped_open_port,
-) -> RuntimeGRPCServer:
+def runtime_grpc_server(session_scoped_open_port) -> RuntimeGRPCServer:
     with runtime_grpc_test_server(
         session_scoped_open_port,
     ) as server:
@@ -161,6 +167,12 @@ def runtime_grpc_server(
         yield server
 
 
+@pytest.fixture(scope="session")
+def model_runtime_servicer(runtime_grpc_server) -> ModelRuntimeServicerImpl:
+    # Builds a new servicer, the one in the server is a bit hard to access
+    return ModelRuntimeServicerImpl(interrupter=runtime_grpc_server.interrupter)
+
+
 @contextmanager
 def runtime_http_test_server(open_port, *args, **kwargs):
     """Helper to wrap creation of RuntimeHTTPServer in temporary configurations"""
@@ -179,12 +191,18 @@ def runtime_http_test_server(open_port, *args, **kwargs):
             },
             "merge",
         ):
-            config_overrides = {}
-            if "tls_config_override" in kwargs:
-                config_overrides = kwargs["tls_config_override"]
-                kwargs["tls_config_override"] = config_overrides["runtime"]["tls"]
+            # Forward the special "tls_config_override" to "tls_config_override"
+            # IFF the configs contain actual TLS (indicated by the presence of
+            # the special "use_in_test" element).
+            config_overrides = kwargs.pop("tls_config_override", {})
+            if tls_config_override := config_overrides.get("runtime", {}).get("tls"):
+                kwargs["tls_config_override"] = tls_config_override
+            else:
+                config_overrides = {}
+            check_readiness = kwargs.pop("check_readiness", True)
             with http_server.RuntimeHTTPServer(*args, **kwargs) as server:
-                _check_http_server_readiness(server, config_overrides)
+                if check_readiness:
+                    _check_http_server_readiness(server, config_overrides)
                 # Give tests access to the workdir
                 server.workdir = workdir
                 yield server
@@ -203,6 +221,17 @@ def runtime_http_server(
         yield server
 
 
+@contextmanager
+def runtime_test_server(*args, protocol: str = "grpc", **kwargs):
+    """Helper function to yield either server"""
+    if protocol == "http":
+        with runtime_http_test_server(*args, **kwargs) as server:
+            yield server
+    elif protocol == "grpc":
+        with runtime_grpc_test_server(*args, **kwargs) as server:
+            yield server
+
+
 @pytest.fixture(scope="session")
 def inference_stub(sample_inference_service, runtime_grpc_server) -> Type:
     inference_stub = sample_inference_service.stub_class(
@@ -267,6 +296,23 @@ def file_task_model_id(box_model_path) -> str:
     model_manager.unload_model(model_id)
 
 
+@pytest.fixture
+def primitive_task_model_id(primitive_model_path) -> str:
+    """Loaded model ID using model manager load model implementation"""
+    model_id = random_test_id()
+    model_manager = ModelManager.get_instance()
+    # model load test already tests with archive - just using a model path here
+    model_manager.load_model(
+        model_id,
+        local_model_path=primitive_model_path,
+        model_type=Fixtures.get_good_model_type(),  # eventually we'd like to be determining the type from the model itself...
+    )
+    yield model_id
+
+    # teardown
+    model_manager.unload_model(model_id)
+
+
 @pytest.fixture
 def sample_task_unary_rpc(sample_inference_service: ServicePackage) -> TaskPredictRPC:
     return sample_inference_service.caikit_rpcs["SampleTaskPredict"]
@@ -383,7 +429,9 @@ class ModuleSubproc:
             self.proc.kill()
 
     def __enter__(self):
-        self.proc = subprocess.Popen(self._cmd, env=self._env)
+        self.proc = subprocess.Popen(
+            self._cmd, env=self._env, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+        )
         self._kill_timer.start()
         return self.proc
 
@@ -417,12 +465,10 @@ def _check_server_readiness(server):
 
 def _check_http_server_readiness(server, config_overrides: Dict[str, Dict]):
     mode = "http"
-    verify = None
     cert = None
     # tls
     if config_overrides:
         mode = "https"
-        verify = config_overrides["use_in_test"]["ca_cert"]
         # mtls
         if "client_cert" and "client_key" in config_overrides["use_in_test"]:
             cert = (
@@ -432,11 +478,13 @@ def _check_http_server_readiness(server, config_overrides: Dict[str, Dict]):
     done = False
     while not done:
         try:
-            response = requests.get(
-                f"{mode}://localhost:{server.port}{http_server.HEALTH_ENDPOINT}",
-                verify=verify,
-                cert=cert,
-            )
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore", module="urllib3")
+                response = requests.get(
+                    f"{mode}://localhost:{server.port}{http_server.HEALTH_ENDPOINT}",
+                    verify=False,
+                    cert=cert,
+                )
             assert response.status_code == 200
             assert response.text == "OK"
             done = True
@@ -445,3 +493,133 @@ def _check_http_server_readiness(server, config_overrides: Dict[str, Dict]):
                 "[HTTP server not ready]; will try to reconnect to test server in 0.01 second."
             )
             time.sleep(0.001)
+
+
+## TLS Helpers #####################################################################
+
+
+@dataobject(package="caikit_data_model.test")
+class KeyPair(DataObjectBase):
+    cert: str
+    key: str
+
+
+@dataobject(package="caikit_data_model.test")
+class TLSConfig(DataObjectBase):
+    server: KeyPair
+    client: KeyPair
+
+
+@contextmanager
+def generate_tls_configs(
+    port: int,
+    tls: bool = False,
+    mtls: bool = False,
+    inline: bool = False,
+    separate_client_ca: bool = False,
+    server_sans: Optional[List[str]] = None,
+    client_sans: Optional[List[str]] = None,
+    **http_config_overrides,
+) -> Dict[str, Dict]:
+    """Helper to generate tls configs"""
+    with tempfile.TemporaryDirectory() as workdir:
+        config_overrides = {}
+        client_keyfile, client_certfile = None, None
+        ca_cert, server_cert, server_key = None, None, None
+        use_in_test = config_overrides.setdefault("use_in_test", {})
+        use_in_test["workdir"] = workdir
+        if mtls or tls:
+            ca_key = tls_test_tools.generate_key()[0]
+            ca_cert = tls_test_tools.generate_ca_cert(ca_key)
+            server_key, server_cert = tls_test_tools.generate_derived_key_cert_pair(
+                ca_key=ca_key,
+                san_list=server_sans,
+            )
+            server_certfile, server_keyfile = save_key_cert_pair(
+                "server", workdir, server_key, server_cert
+            )
+
+            if inline:
+                tls_config = TLSConfig(
+                    server=KeyPair(cert=server_cert, key=server_key),
+                    client=KeyPair(cert="", key=""),
+                )
+            else:
+                tls_config = TLSConfig(
+                    server=KeyPair(cert=server_certfile, key=server_keyfile),
+                    client=KeyPair(cert="", key=""),
+                )
+
+            # need to save this ca_certfile in config_overrides so the tls
+            # tests below can access it from client side
+            ca_certfile, _ = save_key_cert_pair("ca", workdir, cert=ca_cert)
+            use_in_test["ca_cert"] = ca_certfile
+            use_in_test["server_key"] = server_keyfile
+            use_in_test["server_cert"] = server_certfile
+
+            # also saving a bad ca_certfile for a failure test case
+            bad_ca_file = os.path.join(workdir, "bad_ca_cert.crt")
+            with open(bad_ca_file, "w") as handle:
+                bad_cert = (
+                    "-----BEGIN CERTIFICATE-----\nfoobar\n-----END CERTIFICATE-----"
+                )
+                handle.write(bad_cert)
+            use_in_test["bad_ca_cert"] = bad_ca_file
+
+            if mtls:
+                if separate_client_ca:
+                    subject_kwargs = {"common_name": "my.client"}
+                    client_ca_key = tls_test_tools.generate_key()[0]
+                    client_ca_cert = tls_test_tools.generate_ca_cert(
+                        client_ca_key, **subject_kwargs
+                    )
+                else:
+                    subject_kwargs = {}
+                    client_ca_key = ca_key
+                    client_ca_cert = ca_cert
+
+                # If inlining the client CA
+                if inline:
+                    tls_config.client.cert = client_ca_cert
+                else:
+                    client_ca_certfile, _ = save_key_cert_pair(
+                        "client_ca", workdir, cert=client_ca_cert
+                    )
+                    tls_config.client.cert = client_ca_certfile
+
+                # Set up the client key/cert pair derived from the client CA
+                client_certfile, client_keyfile = save_key_cert_pair(
+                    "client",
+                    workdir,
+                    *tls_test_tools.generate_derived_key_cert_pair(
+                        ca_key=client_ca_key,
+                        san_list=client_sans,
+                        **subject_kwargs,
+                    ),
+                )
+                # need to save the client cert and key in config_overrides so the mtls test below can access it
+                use_in_test["client_cert"] = client_certfile
+                use_in_test["client_key"] = client_keyfile
+
+            config_overrides["runtime"] = {"tls": tls_config.to_dict()}
+        config_overrides.setdefault("runtime", {})["http"] = {
+            "server_shutdown_grace_period_seconds": 0.01,  # this is so the server is killed after 0.1 if no test is running
+            "port": port,
+            **http_config_overrides,
+        }
+
+        with temp_config(config_overrides, "merge"):
+            yield aconfig.Config(config_overrides)
+
+
+def save_key_cert_pair(prefix, workdir, key=None, cert=None):
+    crtfile, keyfile = None, None
+    if key is not None:
+        keyfile = os.path.join(workdir, f"{prefix}.key")
+        with open(keyfile, "w") as handle:
+            handle.write(key)
+    if cert is not None:
+        crtfile = os.path.join(workdir, f"{prefix}.crt")
+        with open(crtfile, "w") as handle:
+            handle.write(cert)
+    return crtfile, keyfile
